#ifndef _SMOOTH_H
#define _SMOOTH_H

// Smoothing of data vectors
// (C) 2002 Frank Paehlke <frank@paehlke.de>
// $Id: smooth.h,v 1.3 2004/01/29 09:32:46 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <vector>
#include <stdexcept>

using namespace std;

//! abstract base class for smoothing algorithms

class smooth {
public:
    
    //! Smooth a data vector X0, putting the output into X
    /*! The maximum distance of changed coordinates to their original values
     *  is bound by the parameter epsilon. */

    virtual void smooth_vector
	(const vector<double>& X0, vector<double>& X, double epsilon) const
	throw() = 0;
};
    
//! Smoothing algorithm "minimize 2nd derivative"
/*! This algorithm alters the values of a data vector
 *  x[i] (i=0..n)
 *  such that the target function
 *  tf(X) = sum ((2*x[i] - x[i-1] - x[i+1])^2, i=1..n-1)
 * ("square sum of 2nd derivative") is minimized.
 *
 * The target function can be changed by inheriting from min_d2_smooth
 * and overloading the virtual functions tf(), dtf(), and d2tf() */

class min_d2_smooth : public smooth
{
 public:

    min_d2_smooth(double acc = 1e-5) throw() : accuracy(acc) {} 
    
    void set_accuracy (double acc) throw() { accuracy = acc; }

    double get_accuracy () const throw() { return accuracy; }

    //! Minimize tf(X) under the condition (X[i]-X0[i] <= epsilon).

    /*! tf(X) is minimized using an iterative algorithm until the
     *  change of tf(X) between two subsequent iterations is smaller
     *  than "accuracy" */

    void smooth_vector
	(const vector<double>& X0, vector<double>& X, double epsilon) const
	throw();
    
 protected:

    //! The target function.
    /*! tf(X) = sum ((2*x[i] - x[i-1] - x[i+1])^2, i=1..n-2)
     *  If n < 2, then tf(X) = 0. */
    
    virtual double tf(const vector<double>& X) const throw();
    
    //! partial derivative of target function by X[i]
    
    virtual double dtf(const vector<double>& X, int i) const
	throw(out_of_range);
    
    //! 2nd partial derivative of target function by X[i]
    
    virtual double d2tf(const vector<double>& X, int i) const
	throw(out_of_range);
    
 private:
    
    double accuracy;
};

#endif
